import { defineNuxtModule, createResolver, addImports } from '@nuxt/kit';

const parseFormat = (src) => {
  const [, format] = src.split(/\.(?=[^.]+$)/);
  return format;
};

const generateStyles = (fonts) => {
  let fontFace = "";
  let classes = "";
  let root = "";
  let variables = "";
  for (const font of fonts) {
    const options = {
      weight: "400",
      display: "optional",
      style: "normal",
      ...font
    };
    const local = options;
    const srcFormat = parseFormat(options.src);
    let format = srcFormat;
    if (srcFormat === "ttf")
      format = "truetype";
    else if (srcFormat === "otf")
      format = "opentype";
    const fontFallback = options.fallback ? `,${options.fallback}` : "";
    const fontFamily = `font-family:"${options.family}";`;
    const fontWeight = `font-weight:${options.weight};`;
    const fontDisplay = `font-display:${options.display};`;
    const fontStyle = `font-style:${options.style};`;
    const fontSrc = `src:url('${options.src}') format('${format}');`;
    let unicodes = "";
    let fontUnicode = "";
    if (options.class)
      classes += `.${options.class}{font-family:"${options.family}"${fontFallback};}`;
    if (options.variable)
      variables += `--${options.variable}:"${options.family}"${fontFallback};`;
    if (local.unicode) {
      for (const code of local.unicode)
        unicodes += `${code},`;
      fontUnicode = `unicode-range:${unicodes.replace(/,*$/, "")};`;
    }
    fontFace += `@font-face{${fontFamily}${fontWeight}${fontDisplay}${fontStyle}${fontSrc}${fontUnicode}}`;
  }
  if (variables)
    root += `:root{${variables}}`;
  return {
    fontFace,
    classes,
    root
  };
};

const module = defineNuxtModule({
  meta: {
    name: "nuxt-font-loader",
    configKey: "fontLoader",
    compatibility: {
      nuxt: ">=3.0.0"
    }
  },
  defaults: {
    autoImport: false
  },
  setup(options, nuxt) {
    const { local, external, autoImport } = options;
    const head = nuxt.options.app.head;
    const { resolve } = createResolver(import.meta.url);
    nuxt.options.build.transpile.push(resolve("./runtime"));
    const composables = resolve("./runtime/composables");
    nuxt.options.alias["#font-loader"] = composables;
    if (autoImport) {
      addImports([
        {
          name: "useLocalFont",
          from: composables
        },
        {
          name: "useExternalFont",
          from: composables
        }
      ]);
    }
    if (local) {
      const { fontFace, classes, root } = generateStyles(local);
      const styles = `${fontFace}${classes}${root}`;
      for (const font of local) {
        const options2 = {
          preload: true,
          ...font
        };
        if (options2.preload) {
          const format = parseFormat(font.src);
          head.link?.push({
            rel: "preload",
            as: "font",
            type: `font/${format}`,
            crossorigin: "anonymous",
            href: font.src
          });
        }
      }
      head.style?.push({ children: styles });
    }
    if (external) {
      const { classes, root } = generateStyles(external);
      const styles = `${classes}${root}`;
      let google = false;
      let typekit = false;
      for (const font of external) {
        if (font.src.includes("google") && !google) {
          google = true;
          head.link?.push(
            {
              rel: "preconnect",
              href: "https://fonts.googleapis.com"
            },
            {
              rel: "preconnect",
              crossorigin: "anonymous",
              href: "https://fonts.gstatic.com"
            }
          );
        }
        if (font.src.includes("typekit") && !typekit) {
          typekit = true;
          head.link?.push({
            rel: "preconnect",
            crossorigin: "anonymous",
            href: "https://use.typekit.net"
          });
        }
        head.link?.push(
          {
            rel: "preload",
            as: "style",
            href: font.src
          },
          {
            rel: "stylesheet",
            href: font.src
          }
        );
      }
      if (styles)
        head.style?.push({ children: styles });
    }
  }
});

export { module as default };
